/*
C implementation of the temporal stump optimization
Pedro Ribeiro & Plinio Moreno 2006

/* $Revision: 0.8 $ */

#include "mex.h"

void cicloPhiChi(float *x,float *w,float *z,
				 float *betaVals,float *alphaVals,
				 int Nsamples,int MaxThre,float Tmax,
				 float *phi,float *chi,float *a,
				 float *b,float *J,float *wz)  
{
  int ii,i,j,T,th,NxB,NxL,status,countTth = 0, count = 0 ,countj = 0;
  float wzphiSum,wzchiSum,wphiSum,wchiSum,wphichiSum,jSum,norm;

  count = 0;
  for (i = 0; i < Nsamples; i++) {
		wz[count] = w[count] * z[count];
      count++;
  }

  for (T = 0; T < Tmax; T++){
	  for (th = 0; th < MaxThre; th++){
		  wzphiSum = 0;wzchiSum = 0;
		  wphiSum = 0;wchiSum = 0;
		  wphichiSum = 0;count=0;
		  for (i = 0; i < Nsamples; i++){
		    /*if (T==0 && th == 0)
		      /wz[count] = w[count] * z[count];*/
			  NxB=0;NxL=0;
			  if (i >= T){
				  status = 1;
				  for (j = i-T; j <= i; j++){
						if (z[j] == z[i]){
							if (x[j] > alphaVals[th])
								NxB++;
							else
								NxL++;
						}
						else if (z[j] != z[i])
							status=0;

				}
			  }
			  if (status){
				  phi[count] = (float)NxB/((int)T+1);chi[count] = (float)NxL/((int)T+1);
			  }
			  else{
				  phi[count] = 0;chi[count] = 0;
			  }
			  if (i >= T){
				wzphiSum = wzphiSum + wz[i] * phi[count];
				wzchiSum = wzchiSum + wz[i] * chi[count];
				wphiSum = wphiSum + w[i] * phi[count] * phi[count];
				wchiSum = wchiSum + w[i] * chi[count] * chi[count];
				wphichiSum = wphichiSum + w[i] * phi[count] * chi[count];
			  }
			  count++;
		  }
		  norm = (wphiSum * wchiSum) - (wphichiSum * wphichiSum);
		  a[countTth]=(wzphiSum * wchiSum - wzchiSum * wphichiSum) / norm;
		  b[countTth]=(wphiSum * wzchiSum - wzphiSum * wphichiSum) / norm;
		  jSum = 0;countj=0;
		  for (ii = 0; ii < Nsamples; ii++){
			  jSum = jSum +( w[ii] * (z[ii] - (a[countTth]*phi[countj] + b[countTth]*chi[countj]))*
				  (z[ii] - (a[countTth]*phi[countj] + b[countTth]*chi[countj])));
		  countj++;
		  }
		  J[countTth]=jSum;
		  countTth++;
	  }
  }
}
/*
for T = 1:MaxT
    for thI=1:Nthre
        J(T,thI) = sum(w.*(z - ( a(T,thI) * PHI(:,thI,T)'   + b(T,thI) * CHI(:,thI,T)' ) ).^2 );
    end
end
*/

/* The gateway routine */
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
  float *x, *w,*z,*betaVals,*alphaVals,
			*phi,*chi,*a,*b,*J,*wz;
  float Tmax;
  int status,mrows, Nsamples, MaxThre;
  int size1[2],size2[2],size3[2];
  /* Check the input arguments. 
  if (nrhs != 6) 
    mexErrMsgTxt("6 inputs required.");
  if (nlhs != 8) 
    mexErrMsgTxt("8 output required.");  */
  /*if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||
      mxGetN(prhs[0])*mxGetM(prhs[0]) != 1) {
    mexErrMsgTxt("Input x must be a scalar.");*/

  
  /* Get the scalar input Tmax & Create pointers to 
  the input data - x,w,z, beta, alpha...*/
  Tmax = mxGetScalar(prhs[0]);  
  x = mxGetPr(prhs[1]);
  w = mxGetPr(prhs[2]);
  z = mxGetPr(prhs[3]);
  betaVals = mxGetPr(prhs[4]);
  alphaVals = mxGetPr(prhs[5]);
  /* Get the dimensions of the data input. */
  mrows = mxGetM(prhs[1]);
  Nsamples = mxGetN(prhs[1]);
  MaxThre = mxGetN(prhs[4]);

  /* Set the output pointer to the output matrixs. */
  size1[0]=Nsamples;
  size1[1]=MaxThre*(int)Tmax;
  size2[0]=MaxThre;
  size2[1]=(int)Tmax;
  size3[0]=mrows;
  size3[1]=Nsamples;
  plhs[3] = mxCreateNumericArray(2,size3, mxSINGLE_CLASS, 0);
  plhs[4] = mxCreateNumericArray(2,size3, mxSINGLE_CLASS, 0);
  plhs[0] = mxCreateNumericArray(2,size2, mxSINGLE_CLASS, 0);
  plhs[1] = mxCreateNumericArray(2,size2, mxSINGLE_CLASS, 0);
  plhs[2] = mxCreateNumericArray(2,size2, mxSINGLE_CLASS, 0);
  plhs[5] = mxCreateNumericArray(2,size3, mxSINGLE_CLASS, 0);
  
  /*plhs[3] = mxCreateDoubleMatrix(Nsamples,MaxThre*(int)Tmax, mxREAL);//PHI
  plhs[4] = mxCreateDoubleMatrix(Nsamples,MaxThre*(int)Tmax, mxREAL);//CHI
  plhs[0] = mxCreateDoubleMatrix(MaxThre,(int)Tmax, mxREAL);//WZPHI
  plhs[1] = mxCreateDoubleMatrix(MaxThre,(int)Tmax, mxREAL);//WZCHI
  plhs[2] = mxCreateDoubleMatrix(MaxThre,(int)Tmax, mxREAL);//WPHI
  plhs[5] = mxCreateDoubleMatrix(mrows,Nsamples, mxREAL);//wz*/
  /*plhs[0] = mxCreateDoubleMatrix(mrows,Nsamples, mxREAL);*/
  /* Create a C pointer to a copy of the output matrix. */
  phi		= mxGetPr(plhs[3]);
  chi		= mxGetPr(plhs[4]);
  a			= mxGetPr(plhs[0]);
  b			= mxGetPr(plhs[1]);
  J			= mxGetPr(plhs[2]);
  wz		= mxGetPr(plhs[5]);

  /*cicloPhiChi(wz,w,z,Nsamples);*/
  /* Call the C subroutine. */
 cicloPhiChi(x,w,z,betaVals,alphaVals,Nsamples,MaxThre,Tmax,
	  phi,chi,a,b,J,wz);

}
