function [th, a , b, TOpt, J] = fitTemporalStump(x, z, w , TemporalSupport);
% [th, a , b] = fitRegressionStump(x, z, w, TemporalSupport);
% The regression has the form:
% h_hat(x_(i)) = 1/T * sum_(t=0)^(T-1) (a * (x_(i-t)>th) + b * (x_(i-t)<=th) );
%
% where (a,b,th) are so that it minimizes the weighted error:
% error = sum(w * (z -1/T * sum_(t=0)^(T-1) (a * (x_(i-t)>th) + b * (x_(i-t)<=th) ) )^2)
%
% maximum T = TemporalSupport * Nsamples;
% x,z and w are vectors of the same length
% x, and h are real values.
% z = {-1,1}.
% w is a weight of positive values. There is no asumption that it sums to
% one.

%  Pedro & Plinio, 2006

[Nfeatures, Nsamples] = size(x); % Nsamples = Number of thresholds that we will consider
w = w/sum(w); % just in case...
WZ=w.*z;
[xSorted, j] = sort(x); % this now becomes the thresholds. I assume that all the values are different.


%MODIFY THIS TO USE A FIXED NUMBER OF THRESHOLDS
[thresh qq]=unique(xSorted);
%%%%%%%%%%%5
% minx=xSorted(1);maxx=xSorted(end);
% thresh=minx:(maxx-minx)/50:maxx;
%%%%%%%%%%%%%%%%%%



%for all possible values of T (1, ..., T)
MaxT = TemporalSupport ;%* Nsamples;
Nthre = length(thresh);

[a,b,J]=fitConstantTemporalStumpNew(single(MaxT),single(x),single(w),single(z),single(thresh),single(thresh));
a=a';b=b';J=J';

%find minimum
[vec1 ind1]=min(J);
[vec2 ind2]=min(vec1);
if TemporalSupport >1
    TOpt=ind1(ind2);ThOpt=ind2;
    a=a(TOpt,ThOpt);
    b=b(TOpt,ThOpt);
    th=thresh(ThOpt);
    J=J(TOpt,ThOpt);
else
    TOpt=1;ThOpt=ind1;
    a=a(TOpt,ThOpt);
    b=b(TOpt,ThOpt);
    th=thresh(ThOpt);
    J=J(TOpt,ThOpt);
end;
