function [Cx, Fx , Tmax, hits,phis,chis] = strongGentleTemporalClassifier( x, y, classifier,error_comp)
% [Cx, Fx] = strongLogitClassifier(x, classifier)
%
% Cx is the predicted class 
% Fx is the output of the additive model
% Cx = sign(Fx)
%
% In general, Fx is more useful than Cx.
%
% The weak classifiers are stumps

% Friedman, J. H., Hastie, T. and Tibshirani, R. 
% "Additive Logistic Regression: a Statistical View of Boosting." (Aug. 1998) 

% modified to use temporal window by:
%  Pedro & Plinio, 2006

Nstages = length(classifier);
[Nfeatures, Nsamples] = size(x); % Nsamples = Number of thresholds that we will consider
Tmax=0;
Fx = zeros(1, Nsamples);
for m = 1:Nstages
    k = classifier(m).featureNdx;
    th = classifier(m).th;
    a = classifier(m).a;
    b = classifier(m).b;
    Te = classifier(m).T;   
    if Tmax<Te
        Tmax=Te;
    end    
    for i = Te:Nsamples
        PHI(i) = length( find( x(k,i-Te+1:i) > th ) )/Te;
        CHI(i) = length( find( x(k,i-Te+1:i) <= th ) )/Te;
    end
    phis(m,:)=PHI;
    chis(m,:)=CHI;
    fm = (1) * (a * PHI + b * CHI); % evaluate weak classifier
    Fx = Fx + fm ; %add regression stump
    switch error_comp
        case 1
            %error computation
            Fx(find(Fx>1))=1;Fx(find(Fx<-1))=-1;
            hits(m) =1/(Nsamples-Tmax-1)*sum(abs((y([Tmax:end]) - Fx([Tmax:end]))/2));
        case 0
            %Recognition rate computation
            Cx=sign(Fx);
            hits(m) = length(find(Cx([Tmax:end]).*y([Tmax:end])>0))/length(Cx([Tmax:end]));
    end 
end
Cx = sign(Fx);
